/* ========================================================================= //
# vi:  set ts=4:
# vim: set ts=4:
#?
#? NAME
#?      EnDeUser.js
#?
#? SYNOPSIS
#?      <SCRIPT language="JavaScript1.3" type="text/javascript" src="EnDeGUI.js"></SCRIPT>
#?      <SCRIPT language="JavaScript1.3" type="text/javascript" src="EnDeUtil.js"></SCRIPT>
#?      <SCRIPT language="JavaScript1.5" type="text/javascript" src="EnDeUser.js"></SCRIPT>
#?
#? DESCRIPTION
#?      This file contains user defined functions/methods used in  EnDe.js .
#?      It defines the  EnDeUser  class with following functions:
#?          EN.vs      - fold viewstate (ASP.NET 1.x only, 2.x experimental)
#?          DE.vs      - parse viewstate (ASP.NET 1.x only, 2.x experimental)
#?          XML        - data structure for reading XML data
#?          XML.menu   - build a SELECT menu from given XML data
#?          XML.core   - build a SELECT menu for core-rules.xml
#?          XML.read   - read XML or JSON data from file and build menu
#?          init       - initialize loading of user files
#?          Check.dispatch - dispatcher for checksum functions
#?          Check.quess- guessing checksums
#?          IMG        - data structure for reading image data
#?          IMG.guess  _ try to identify image format
#?          IMG.getMIME64  - get MIME type prefix for base64 encoded image data
#?
#? PRE-REQUESTS
#?      Some functions herein rely on variables and functions set elsewhere:
#?          EnDeGUI.alert()
#?          EnDeGUI.onClick
#?          EnDe.isKonqueror
#?
#?      If these they are missing, somehow, they can simply be implemented as
#?      follows (which is the default anyway):
#?          EnDeGUI.alert    = function(f,s){alert(f+':'+s);}
#?          EnDeGUI.onClick  = true;
#?          EnDe.isKonqueror = false;
#?
#? SEE ALSO
#?      EnDe.js
#?      EnDeGUI.js
#?      EnDeFile.xml
#?      EnDeSamp.xml
#?
# HACKER's INFO
#       Simple debugging available if  userdebug  given in QUERY_STRING.
#?
#? VERSION
#?      @(#) EnDeUser.js 1.60 09/10/29 22:43:48
#?
#? AUTHOR
#?      27-nov-07 Achim Hoffmann, mailto: EnDe (at) my (dash) stp (dot) net
#?
 * ========================================================================= */

// ========================================================================= //
// Extension to EnDeGUI object                                               //
// ========================================================================= //
EnDeGUI.opts   = new Array(); // used by EnDeGUI.preset()
// this array will be generated while reading  .xml  file, see this.menu below
// the generated array looks as follows:
/* ------------------------+-------+-------+-------+-------+-------+-------+---- */
/*            option        size    upper   ishex   prefix  suffix delimiter key */
/* ------------------------+-------+-------+-------+-------+-------+-------+----
EnDeGUI.opts['MxSQL_1'] = [ '1',    false,  false,  'CHAR(',')',    '+',    null ];
   ------------------------+-------+-------+-------+-------+-------+-------+---- */
EnDeGUI.optLen =            0;
EnDeGUI.optCas =                    1;
EnDeGUI.optHex =                            2;
EnDeGUI.optPre =                                    3;
EnDeGUI.optSuf =                                            4;
EnDeGUI.optDel =                                                    5;
EnDeGUI.optKey =                                                            6;
/**
 ** NOTE
 ** definition of above EnDeGUI.* attributes requires that this EnDeUser.js
 ** must be included after  EnDeGUI.js  otherwise some browsers (i.e. Mozilla)
 ** will throw an exception and/or an error: "EnDeGUI.opts has no properties"
 **/

// ========================================================================= //
// EnDeUser object methods                                                   //
// ========================================================================= //

var EnDeUser = new function() {
this.SID    = function() { return('@(#) EnDeUser.js 1.60 09/10/29 22:43:48 EnDeUser'); };

	// ===================================================================== //
	// global variables                                                      //
	// ===================================================================== //

this.delay  = 1000; // 1 second, used for setTimeout()
this.ident  = '|   ';
this.pos    = 0;	// need a global, ugly ...
this.debug  = 0;

	// ===================================================================== //
	// Mozilla privilege Manager                                             //
	// ===================================================================== //

/*
	UniversalFileAccess (probably Netscape only),
	UniversalFileRead,        UniversalFileWrite,
	UniversalBrowserRead,     UniversalBrowserWrite,
	UniversalPreferencesRead, UniversalPreferencesWrite
 for details see:
	http://www.mozilla.org/projects/security/components/signed-script-example.html
 */
this.setPriv= function(src) {
//#? set browser privileges
	if (document.location.protocol.match(/file:/)==null) {
		EnDeGUI.alert('EnDeUser.setPriv', "privileges can be set when used with 'file://' only; file read failed");
		return false;
	}
	if (!netscape.security.PrivilegeManager.enablePrivilege) {    return false; }
	try {netscape.security.PrivilegeManager.enablePrivilege(src); return true;  }
	catch(e) { EnDeGUI.alert('EnDeUser.setPriv', e); return false; }
	return false;
};

	// ===================================================================== //
	// misc. methods                                                         //
	// ===================================================================== //

	// ===================================================================== //
	// viewstate functions                                                   //
	// ===================================================================== //

this.EN     = new function() {
  this.SID  = function() { return(EnDeUser.SID() + '.EN'); };
  this.vs   = function(src) {
  //#? convert data to ViewState 1.x format
	src = src.replace(/\|   /g,'');
	src = src.replace(/\n/g,'');
	return src;
  };
}; // EN

this.DE     = new function() {
  this.SID  = function() { return(EnDeUser.SID() + '.DE'); };
  this.vs   = function(src) {
  //#? convert data from ViewState 1.x format
	if (EnDe.isKonqueror){
		EnDeGUI.alert('** Sorry, Konqueror cannot use this function **');
		return src;
	}
	function _fff1(src) {
		// unicodes fff1 .. fffb are not used (in most browsers, hopefully),
		// hence we use them to mark own insertions
		return String.fromCharCode(65520+src);
	}
	function _mask(src) {
		// unicodes fff1 .. fffb are not used (in most browsers, hopefully),
		// hence we use them to mark own insertions
		switch (src) {
			  case 65520 : return(_fff1(1)  + 'Viewstate v1.0' + _fff1(11)); break;
			  case 65521 : return(_fff1(1)  + 'Viewstate v2.0' + _fff1(11)); break;
			  case '\x02': return(_fff1(2)  + 'Controlstate'   + _fff1(11)); break;
			  case '\x04': return(_fff1(4)  + 'Integer'        + _fff1(11)); break;
			  case '\x05': return(_fff1(5)  + 'String'         + _fff1(11)); break;
			  case '\x1e': return(_fff1(5)  + 'string'         + _fff1(11)); break; // ToDo: unsure
			  case '\x0f': return(_fff1(6)  + 'Pair'           + _fff1(11)); break;
			  case '\x16': return(_fff1(3)  + 'List'           + _fff1(11)); break;
			  case '\x17': return(_fff1(7)  + 'Arraylist'      + _fff1(11)); break;
			  case '\x18': return(_fff1(8)  + 'Hashlist'       + _fff1(11)); break;
			  case '\x66': return(_fff1(4)  + 'Int32'          + _fff1(11)); break; // ToDo: unsure
			  case '#'   : return(_fff1(10) + 'MAC'            + _fff1(11)); break;
			  case '['   : return(_fff1(10) + '['              + _fff1(11)); break;
			  case ']'   : return(_fff1(10) + ']'              + _fff1(11)); break;
			  default    : return(_fff1(9)  +  src             + _fff1(11)); break;
		}
		return src; // fallback
	}
	function _ident(i) { var kkk = ''; for (var k=0; k<i; k++, kkk += '|   '){}; return '\n'+kkk; }
	function _bytes(src,i,len) { return src.substr(i,len); }
	function _v1(src,i) {
		// parse Viewstate 1.x format
		// this code is build on reverse engeneering sample data
		var lev = 0;
		var bbb = '';
		var ccc = '';
		var kkk = '';
		var arr = null;
		var rex = null;
		var esc = 0;    // set to 1 if \ found, reset at next character
		var key = 0;    // incremented if < or ; decremented for [tpli]  and >
	                	// used to beautify idents
	    	key = 1;    // we assume that the viewstate's first character is a key
		while (i<src.length) {
			if (i>99999) { EnDeGUI.alert('viewstate too large'); return kkk; break; }
			ccc = src.charAt(i);
			i++;
			if (esc==1) {
				esc = 0;
				kkk += ccc;
				continue;
			}
			switch (ccc) {
			  case 'l'   :
			  case 't'   :
			  case 'p'   :
			  case 'i'   :
				if (key>0) {  // got a special key (tripple, tuplet)
					kkk += '\n' + bbb + ccc;
					key = -1;
				} else {
					kkk += ccc;
				}
				break;
			  case '\\': esc = 1; kkk += ccc; break;
			  case '<':
				lev++;
				if (key>=0) { kkk += '\n' + bbb; }
				//kkk += '\n' + bbb + ccc;
				kkk += ccc;
				key=1;
				/*
				 * limited JavaScript in Firefox (2.0.0.10), returns error
				 *    Error: overlarge minimum (67395) ....
				 * hence no sophisticated compact parsing for huge data
				 */
				if (src.length < 67390 ) {
					rex = new RegExp( '(?:^.{'+i+'})([a-zA-Z0-9.,;:/\(\)\\_-]+>)' );
					arr = src.match(rex);
					if (arr!=null) {
						kkk += arr[1];
						i += arr[1].length;
						arr.length = 0;
					} else {
						bbb += '|   ';
					}
				} else {
					bbb += '|   ';
				}
				break;
			  case '>':
				key=0;
				lev--;
				bbb  = bbb.substr(0,(bbb.length-4));
				kkk += '\n' + bbb + ccc;
				break;
			  case ';':
				key++;
				//kkk += '\n' + bbb + ccc;
				kkk += ccc;
				break;
			  default: kkk += ccc; key = 0; break;
			}
		}
		rex = null;
		return kkk;
	} // _v1

	function _v2(src,key,lev) {
	/* parse src starting at EnDeUser.pos */
		// parse Viewstate 2.x format
		// this ugly code is build on reverse engeneering sample data
		// ToDo: _v2() still incomplete and buggy
		var kkk = '';
		var bbb = '';
		var ccc = 0; // count items in list, hashlist, arraylist, ...
		//#dbx if (lev>1){ alert(EnDeUser.pos+' \n'+src); };
		goab: while (EnDeUser.pos<src.length) {
			/* loop over data stream src (the binary viewstate)
			 * parsed data objects where each object will be identified by
			 * a leading byte (see switch statement) which is followed by the
			 * size of the object
			 * all objects can be used recursive
			 * base object types are "string" and "integer" (probably "byte")
			 */
			if (EnDeUser.pos>199999) { EnDeGUI.alert('viewstate too large'); return bbb; break; }
			switch (src[EnDeUser.pos]) {
/*
	case '\xff\x01':  // format 2.x viewstate
	case '\x01':  // t  - (Triplet) viewstate (sollte nur nach \xff komment)
	case '\x02':  // c  - controlstate
	case '\x03':  // l ??
	case '\x04':  // i 2-byte ??
	case '\x05':  // s  - naechstes Byte ist Laenge (in Bytes) ?? (in Hashtable Anzahl Name=Value Pairs)
	case '\x09':  //   ??
	case '\x0a':  //   ??
	case '\x0c':  //   ??
	case '\x0f\x0f':  // p  - pair
	case '\x0f\x0f\x64':  // MAC ??
	case '\x1b':  //   ??
	case '\x1e':  // ?  - ?unknown String? (naechstes Byte ist Laenge in Bytes)
	case '\x1f':  //   ??
	case '\x16':  //   ?? List - naechstes Byte Anzahl Eintraege
	case '\x17':  // a  - Arraylist (naechstes Byte Anzahl Elemente)
	case '\x18':  // h  - Hashlist
	case '\x63':  //   ??
	case '\x64':  //   ?? (Ende String?) Start MAC?
*/
	
			  case '\x64': bbb += _ident(lev) + _mask(src[EnDeUser.pos]); break;
			  case '\x02': bbb += _ident(lev) + _mask(src[EnDeUser.pos]); break;
			  case '\x04':
						// ToDo: integer probably 2 bytes fixed
			  case '\x1e': // unknown string (next byte is length)
			  case '\x05': // string (next byte is length)
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]);
					EnDeUser.pos++;
					ccc = new Number(src.charCodeAt(EnDeUser.pos));
					bbb += '(' + ccc + ')';
					EnDeUser.pos++;
					kkk  = _bytes(src,EnDeUser.pos,ccc); EnDeUser.pos += kkk.length - 1;
					bbb += _ident(lev+1) + kkk;
					ccc = null;
					break;
			  case '\x0f': // pair
					EnDeUser.pos += 2;
					if (src[EnDeUser.pos] != '\x64') {
						bbb += _ident(lev) + _mask('\x0f'); // got a simple pair
						lev++;
						EnDeUser.pos--;
					} else { // seems to be final MAC
						bbb += _ident(lev) + _mask('#');
						bbb += _ident(lev) + '|' + _mask('[');
						lev++;
						EnDeUser.pos++;
						bbb += _v2(src,1,lev);
						bbb += _ident(lev) + '|' + _mask('[');
						bbb += _ident(lev+1) + _bytes(src,EnDeUser.pos,src.length);
						EnDeUser.pos = src.length;
						bbb += _ident(lev) + '|' + _mask(']');
						lev--;
						bbb += _ident(lev) + '|' + _mask(']');
					}
					if (key==1) { return bbb; } // don't continue loop
					break;
			  case '\x16': // list
			  case '\x17': // array (next byte number of elements)
					ccc = new Number(src.charCodeAt(EnDeUser.pos+1));
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]) + '(' + ccc + ')';
					EnDeUser.pos += 2;
					//EnDeUser.pos += 3;
/*
					EnDeUser.pos++;
					ccc = new Number(src.charCodeAt(EnDeUser.pos));
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]) + '(' + ccc + ')';
					EnDeUser.pos++;
*/
					lev++;
					for (var x=0; x<ccc; x++) {
						bbb += _ident(lev-1) + '|' + _mask('[') + (x+1);
						bbb += _v2(src,1,lev);
						bbb += _ident(lev-1) + '|' + _mask(']');
					}
					lev--;
					ccc = null;
					if (key==1) { return bbb; } // don't continue loop
					continue; // read next byte
					break;
			  case '\x18': // hash
					ccc = new Number(src.charCodeAt(EnDeUser.pos+1));
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]) + '(' + ccc + ')';
					EnDeUser.pos += 2;
					lev++;
					for (var x=0; x<ccc; x++) {
						bbb += _ident(lev-1) + '|' + _mask('{') + (x+1);
						bbb += _v2(src,1,lev);
						bbb += _v2(src,1,lev);
						bbb += _ident(lev-1) + '|' + _mask('}');
					}
					ccc = null;
					if (key==1) { return bbb; } // don't continue loop
					continue; // read next byte
					break;
			  default:     bbb += src[EnDeUser.pos]; _dpr('def:'); break;
			} // switch
			EnDeUser.pos++;
			if (key==1) { return bbb; } // don't continue loop
		}  // while
		return bbb;
	} // _v2
	var net = 0;    // version 1.x or version 2.x
	var ccc = 0;
	var lev = 0;
	var bux = '';
	while (ccc<src.length) { // detect viewstate, silently ignore anything before
		if (ccc>99999) { EnDeGUI.alert('viewstate too large'); break; }
		switch (src[ccc]) {
		  case '\uff01':                                    net = 2;    break;
		  case '\xff'  : if (src[ccc+1] == '\x01') { ccc++; net = 2; }; break; 
		  case 't'     : if (src[ccc+1] == '<')    { ccc++; net = 1; }; break; 
		}
		ccc++;
		if (net != 0) { break; } // found start of viewstate
		bux += src[ccc-1];
	}
	if (net != 1) {
		bux += _mask(65519+net);
		EnDeUser.pos = ccc;
	}
	if (net == 1) { bux += 't<\n' +_v1(src,ccc); } // t<  stripped above ..
	if (net == 2) { bux += _v2(src,0,0); }
	return bux;
  };
}; // DE

	// ===================================================================== //
	// functions to generate Cheat Seat menu                                 //
	// ===================================================================== //

  this.setObj   = function(src,obj) {
  //#? write selected value to specified textarea; **for internal use only**
// ToDo: $(obj).value = EnDeGUI.FF.update(item,mode,uppercase,src,prefix,suffix,delimiter,'EN');
	/* Cannot use EnDeGUI.FF.update() as passed src is complete function call,
	 * see EnDeFunc.xml. We also get all different kund of functions, not only
	 * EnDe.EN.* or EnDe.DE.* .
	 */
	if ($('EnDe.OPT.append').checked==true) {
		$(obj).value += src;
	} else {
		$(obj).value  = src;
	}
  };

  this.setEN    = function(src) { this.setObj(src, 'EnDe.EN.text'); }
  //#? write selected value to Encoding textarea

  this.setDE    = function(src) { this.setObj(src, 'EnDe.DE.text'); }
  //#? write selected value to Decoding textarea

  this.setFF    = function(src) { this.setObj(src, 'EnDe.FF.text'); }
  //#? write selected value to Function textarea

  this.setRE    = function(src) { this.setObj(src, 'EnDe.RE.text'); }
  //#? write selected value to RegEx textarea

  this.setfile  = function(src) { $('EnDe.FT.XMLfile').value = src; };
  //#? write selected value to XMLfile input field

this.XML    = new function() {
  this.SID  = function() { return(EnDeUser.SID() + '.XML'); };
  this.reset= function() {
  //#? reset internal variables, mainly for debugging purpose ..
	EnDeUser.XML.fil = null;
	EnDeUser.XML.use = null;
	EnDeUser.XML.cnt = 3;
	return false;
  }; // XML.reset

  this.use  = null; // filename of currently processed file
  this.fil  = null; // DOM object with content of file
//  this.data = null; // content of read file
  this.cnt  = 3;    // counter to avoid looping
  var files = {
	// target object for menu, will be filled from EnDeFile.xml
	// the static entries are those for EnDe's menus itself 
	/* -------------+------------+-------------------------------+--------------------------+-------------- */
	/* source file  :   target    function to be called           text for label            text for menu button
	/* -------------+------------+-------------------------------+--------------------------+-------------- */
	'EnDeFile.xml'  : ['fileFT', 'EnDeUser.setfile(this.value)', '&#160;File to load: ',    'build menu from this file' ],
	'EnDeOpts.xml'  : ['OPtextMenu', 'EnDeGUI.preset(EnDe,this.value)', '',                 'preset API Options'        ],
	'EnDeFunc.xml'  : ['fsetST', 'EnDeUser.setFF(this.value)',   '&#160;EnDe functions: ',  'EnDe function call'        ],
	'EnDeUser.xml'  : ['fsetST', 'EnDeUser.setFF(this.value)',   '&#160;User functions: ',  'User defined function call']
	/* -------------+------------+-------------------------------+--------------------------+-------------- */
	// more such lines added by this.menu() when reading EnDeFile.xml
  };
	/* Note about the XML format in above files:
	 * the format was originaly used in  http://ha.ckers.org/xssAttacks.xml
	 * and (probably) adopted in http://mario.heideri.ch/xss.xml
	 * we use the same format in  EnDeFile.xml  and our own files so we can
	 * use the same function (this.menu(), see below) to build the menu.
	 */

  this.fileDefault = function(src) {
  //#? add default entry to files{}
	files[src] = ['fsetEN', 'EnDeUser.setEN(this.value)', '&#160;(not configured)', 'user data from file: '+src];
  }; // XML.fileDefault

  this.formatLabel = function(src) {
  //#? return a fromated text for labels
// ToDo: should use DOM instead of innerHTML
	return '<span class="em9">' + src + '</span>';
  }; // XML.formatLabel

  this.read = function(src) {
  //#? read XML or JSON data from file and build menu
	_spr('EnDeUser.XML.load('+src+')');
	EnDeUtil.file.reset();
	EnDeUser.XML.use = src;
	switch (src) {
	  case 'default_filter.json':
		EnDeUtil.file.readTXT(src);
		this.fil = EnDeUtil.file.content;
		this.phpids();
		break;
	  case 'core-rules.xml' :
	  case 'core-rules-2.0.xml' :
		EnDeUtil.file.readXML(src);
		this.fil = EnDeUtil.file.content;
		this.core();
		break;
	default:
		EnDeUtil.file.readXML(src);
		this.fil = EnDeUtil.file.content;
		this.menu();
		break;
	}
	EnDeUtil.file.reset();
	return false;
  }; // XML.load

  this.menu = function() {
  //#? Build a SELECT menu from given XML data.
  /* Data looks like:
	  <xss><attack>
		<name /><code /><label /><desc /><browser />
		<target /><func /><prefix /><opts />
	  </attack></xss>
   */
	if (EnDeUser.debug != 0) {
		_dpr('EnDeUser.XML.menu: =============={ '+EnDeUser.XML.use);
		if (EnDeUser.XML.fil == null) {
			_dpr('EnDeUser.XML.menu: EnDeUser.XML.fil==null');
			return;
		}
		if (EnDeUser.XML.fil == undefined) {
			_dpr('EnDeUser.XML.menu: EnDeUser.XML.fil==undefined');
			return;
		}
	}
	var bux = EnDeUser.XML.fil.getElementsByTagName('attack');
	if (bux.length == 0) {     _dpr('EnDeUser.XML.menu: empty file or read failed; return }'); return; }
	var click = 'onClick';
	try {
	if (EnDeGUI.onClick==false) { click  = 'onChange'; }
	var label = '';
	var txt   = '';
	var arr   = null;
	var bbb   = document.createElement('SELECT');
	var ccc   = document.createElement('OPTGROUP');
	if (files[EnDeUser.XML.use] == undefined) { // file not specified in  EnDeFile.xml
		// simply add it on the fly ..
		EnDeUser.XML.fileDefault(EnDeUser.XML.use);
		files[EnDeUser.XML.use][0] = $('fileFT.obj').value;// ToDo: should not use HTML DOM here
	}
	//#dbx# _dpr('EnDeUser.XML.menu: title="' + files[EnDeUser.XML.use][3] + '"');
	//#dbx# _dpr('EnDeUser.XML.menu: click="' + files[EnDeUser.XML.use][1] + '"');
	bbb.setAttribute('title', files[EnDeUser.XML.use][3]); //#'add Pattern to Encoding textarea');
	bbb.setAttribute( click,  files[EnDeUser.XML.use][1]); //#'EnDeUser.setEN(this.value)');
	var a0 = '';
	var a1 = '';
	var a2 = '';
	var a3 = '';
	var c  = 0;
	for (c=0;c<bux.length;c++) {
		if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.menu[ '+c+' ]'); }
// ToDo: following 2 commands fail in Fiefox 3.x due to permission problems
// see: https://bugzilla.mozilla.org/show_bug.cgi?id=434522; solution unknown (22-jul-08)
// https://bugzilla.mozilla.org/show_bug.cgi?id=424875
/*
 * following works:
 *    user_pref("capability.policy.default.Element.firstChild",  "allAccess");
 * does not work:
 *    user_pref("capability.policy.default.Element.nodeType",    "allAccess");
 */
		try {
		if (bux[c].childNodes.length == 0) continue; // skip empty nodes
		} catch(e){ _dpr('EnDeUser.XML.menu: 0 bux['+c+'].childNodes.length failed: '+e); };
		if (bux[c].nodeType          != 1) continue; // skip empty or text nodes
		// set defaults, see files variable above  // ToDo: this is for EnDeFile.xml only
		a0 = 'fsetEN';                      // target object
		a1 = 'EnDeUser.setEN(this.value)';  // event function
		a2 = '&#160;: ';                    // label
		a3 = '';                            // description/title
		var kkk   = document.createElement('OPTION');
		for (var n=0;n<bux[c].childNodes.length;n++) {
			if (bux[c].childNodes[n].nodeType != 1) continue; // skip empty nodes
			if (bux[c].childNodes[n].firstChild != null) {    // avoid error
				txt = bux[c].childNodes[n].firstChild.nodeValue;
			} else {
				txt = '';
			}
			if (EnDeUser.debug == 2) { _dpr('EnDeUser.XML.menu.line:<'+bux[c].childNodes[n].nodeName+'>'+txt); }
			switch (bux[c].childNodes[n].nodeName) {
			  case 'name':  kkk.text  = txt; break;
			  case 'code':  kkk.value = txt; break;
			  case 'label':
				if (label != txt) {
					label= txt;
					ccc  = document.createElement('OPTGROUP');
					ccc.setAttribute('label', label);
					bbb.appendChild(ccc);
				}
				if (EnDeUser.debug == 2) { _dpr('EnDeUser.XML.menu.label:'+label); }
				break;
			  case 'target': a0 = txt; break;
			  case 'func':   a1 = txt; break;
			  case 'prefix': a2 = txt; break;
/* this is dangerous, so we don't allow eval for arbitrary data, see 'opts' instead
 *			  case 'eval':
 *				try {     eval(txt); }
 *				catch(e){ EnDeGUI.alert('EnDeUser.XML.menu: ['+c+'] eval('+txt+') failed',e); };
 *				break;
 */
			  case 'opts':  // data for EnDeGUI.opts[] array, value found in 'code' is index
				EnDeGUI.opts[kkk.value] = new Array();
				var val = null;
				arr  = txt.split('\t');
				for (var j=0;j<arr.length;j++) {
					// anything we get is a string, check for some special keywords
					switch (arr[j]) {
					  case 'null':  val = null;   break;
					  case 'true':  val = true;   break;
					  case 'false': val = false;  break;
					  default:      val = arr[j]; break;
					}
					EnDeGUI.opts[kkk.value].push(val);
				}
				if (EnDeUser.debug == 2) { _dpr('EnDeUser.XML.menu:EnDeGUI.opts['+kkk.value+']='+EnDeGUI.opts[kkk.value].join(' - ')); }
				arr.length = 0;
				val = null;
				break;
			  case 'desc':
				if (EnDeUser.XML.use == 'EnDeFile.xml' ) { // ToDo: ugly workaround
					a3 = txt;
				} else {
					kkk.title = txt;
// ToDo: blame browser vendors
					if ((EnDeGUI.isWebKit==false) && (EnDeGUI.isSafari==false) && (EnDeGUI.isCamino==false)) {   // they violate w3c
						kkk.label = txt;     // this is what w3c defines, but no browser supports it :-((
					}
				}
				break;
			  default: /* silently ignored */
				if (EnDeUser.debug == 2) { _dpr('EnDeUser.XML.menu.unknown:'+bux[c].childNodes[n].nodeName); }
				break;
			}
		}
		if (EnDeUser.debug != 0) {
			try {
			_dpr('EnDeUser.XML.menu:name='+kkk.text);
			_dpr('EnDeUser.XML.menu:code='+kkk.value);
			} catch(e){ EnDeGUI.alert('EnDeUser.XML.menu: _dpr failed: ',e); }
		}
		if (EnDeUser.XML.use == 'EnDeFile.xml' ) { // update data structure
//alert('txt:'+EnDeUser.XML.use);
			files[kkk.value] = [ a0, a1, a2, a3 ]; // ugly that 'code' is the filename
			if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.menu: ['+kkk.value+'] '+files[kkk.value]); }
		}
		ccc.appendChild(kkk);
	}
	bbb.appendChild(ccc);
	var div = document.createElement('DIV');
	bbb.setAttribute('id',    EnDeUser.XML.use.toString());
	bbb.selectedIndex = -1;  // onClick hack
	div.innerHTML = EnDeUser.XML.formatLabel( files[EnDeUser.XML.use][2] );
	div.className = 'bottom';
	div.appendChild(bbb);
	$(files[EnDeUser.XML.use][0]).appendChild(div);
	//bux.length = 0; // returns: setting a property that has only a getter
	} // try
	catch(e){ EnDeGUI.alert('EnDeUser.XML.menu: failed['+c+']',e); }
	finally { // reset always, also if file contained errors
		bux = null;
		EnDeUser.XML.use = null;
		EnDeUser.XML.cnt = 3;
	}
	if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.menu: ==================}'); }
  }; // XML.menu

  this.core = function() {
  //#? Build a SELECT menu from given XML data for ModSecurity's core rule set.
  /*
	* the core-rules.xml  generated by  CoreRule2HTML.jar  has its own tags
	* which are not comparable to those used in other files (see this.menu)
	* it also has some scopes with missing data which needs to be completed
	* and finally it is not sorted, hence we have to sort it first before
	* createing the menu
	* that's why this function is different to this.menu)() above
  /* Data looks like:
	<core-rules>
		<rule>
			<operator />  // code
			<comment />   // desc
			<selector />  // label
			<action> <msg /><tag /></action>
		</rule>
	</core-rules>
   */
	if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.core: =================={ '+EnDeUser.XML.use); }
	var bux = EnDeUser.XML.fil.getElementsByTagName('rule');
	if (bux.length == 0) { return; } // empty file or read failed
	var click = 'onClick';
	try {
	if (EnDeGUI.onClick==false) { click  = 'onChange'; }
	var label = '';
	var txt   = '';
	var msg   = '';
	var arr   = new Array();
	var labels= new Array();
	var bbb   = document.createElement('SELECT');
	var ccc   = document.createElement('OPTGROUP');
	if (files[EnDeUser.XML.use] == undefined) { // file not specified in  EnDeFile.xml
		// simply add it on the fly ..
		EnDeUser.XML.fileDefault(EnDeUser.XML.use);
		files[EnDeUser.XML.use][0] = $('fileFT.obj').value;
	}
	bbb.setAttribute('title', 'pattern for ModSecurity core rule set');
	bbb.setAttribute( click,  'document.getElementById("EnDe.RE.text").value = this.value;');
	var c  = 0;
	var n  = 0;
	var kkk= null;
	for (c=0;c<bux.length;c++) {
		if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.core[ '+c+' ]'); }
		if (bux[c].childNodes.length == 0) continue; // skip empty nodes
		if (bux[c].nodeType          != 1) continue; // skip empty or text nodes
		// set defaults, see files variable above  // ToDo: this is for EnDeFile.xml only
		msg= '';
		kkk= document.createElement('OPTION');
		for (n=0;n<bux[c].childNodes.length;n++) {
			if (bux[c].childNodes[n].nodeType != 1) continue; // skip empty nodes
			if (bux[c].childNodes[n].firstChild != null) {    // avoid error
				txt = bux[c].childNodes[n].firstChild.nodeValue;
			} else {
				txt = '';
			}
			if (EnDeUser.debug == 2) { _dpr('EnDeUser.XML.core .file.line:<'+bux[c].childNodes[n].nodeName+'>'+txt); }
			switch (bux[c].childNodes[n].nodeName) {
			  case 'comment':   kkk.title = txt; kkk.text = txt; break;
			  case 'operator':  kkk.value = txt; break;
			  case 'name':      kkk.text  = txt; break;
			  case 'selector':  label     = txt; break;
			  default: /* silently ignored */
				if (EnDeUser.debug == 2) { _dpr('EnDeUser.XML.core unknown:'+bux[c].childNodes[n].nodeName); }
				break;
			  case 'actions':
				for (var a=0;a<bux[c].childNodes[n].childNodes.length;a++) {
					switch (bux[c].childNodes[n].childNodes[a].nodeName) {
					  case 'msg' : msg = bux[c].childNodes[n].childNodes[a].firstChild.nodeValue; break;
					//default: silently ignore for now
					}
				}
				break;
			} // switch
		} // nodes
		if (EnDeUser.debug != 0) {
			try {
			_dpr('EnDeUser.XML.core:name='+kkk.text);
			_dpr('EnDeUser.XML.core:code='+kkk.value);
			} catch (e) {}
		}
		// need text and title; use title or operator if missing
		if (kkk.text == '') {
			if (msg != '') {
				kkk.text  = msg;
				kkk.title = msg;
			} else {
				if (kkk.title != '') {
					kkk.text = kkk.title;
				} else {
					kkk.text  = label;
					kkk.title = label;
				}
			}
		}
		// some cleanup ..
		kkk.title = kkk.title.replace(/[\r\n]/g, ' ');
		if (label.length > 58) {
			label = label.substr(0,58) + ' ...';
		}
		kkk.text = kkk.text.substr(0,60) // use stripped comment
		// .. and push OPTION object on the stack
		if ((arr[label] == null) || (arr[label] == undefined)) {
			labels.push(label);
			arr[label] = new Array();
		}
		arr[label].push(kkk);
	}
	// now build the menu
	/* as we loop through the hash, it's automatically grouped correctly */
	for (n in labels) {
		label= labels[n];
		ccc  = document.createElement('OPTGROUP');
		ccc.setAttribute('label', label);
		while((obj = arr[label].pop())!=null) {
			ccc.appendChild(obj);
		}
		bbb.appendChild(ccc);
	}
	var div = document.createElement('DIV');
	bbb.setAttribute('id',    EnDeUser.XML.use.toString());
	div.innerHTML = EnDeUser.XML.formatLabel( files[EnDeUser.XML.use][2] );
	div.className = 'bottom';
	div.appendChild(bbb);
	$(files[EnDeUser.XML.use][0]).appendChild(div);
	} // try
	catch(e){ EnDeGUI.alert('EnDeUser.XML.core: failed['+c+']',e); }
	finally { // reset always, also if file contained errors
		bux = null;
		EnDeUser.XML.use = null;
		EnDeUser.XML.cnt = 3;
	}
	if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.core: ==================}'); }
  }; // XML.core

  this.phpids   = function() {
  //#? Build a SELECT menu from given JSON data for PHPIDS rules.
	/* Data looks like:
	{"filters":{
		"filter":[
			{"id":         "1",
			 "rule":       "regex here",
			 "description":"descriptive text",
			 "tags":       {"tag":["xss","csrf"]},
			 "impact":     "4"
			},
			{"id":"2", .. },
			 .. 
		}
	}
	*/
	// read data and convert to JavaScript code suitable vor eval()
	if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.phpids: ============{ '+EnDeUser.XML.use); }
	var bux = '';
	var kkk = EnDeUser.XML.fil.replace(/[\n\r]/g, ''); // eval() is picky with newlines
	if (kkk.match(/^\s*\{/) != null) {
		// at least the beginning is raw JSON data, assign it to own variable
		kkk = 'bux=' + kkk + ';'; // semicolon at very end just in case of ..
	} else {
		// just a try, should not happen with PHPIDS data
		kkk = kkk.replace(/^\s*[a-zA-Z0-9]+\s*=/, 'kkk=');
	}
	var click = 'onClick';
	if (EnDeGUI.onClick==false) { click  = 'onChange'; }

	// eval code and build DOM objects from its data
//alert('#\n'+kkk+'\n#');
	try     { eval(kkk); }
	catch(e){ EnDeGUI.alert('EnDeUser.XML.phpids: eval failed',e); }
	kkk = null;

	try {
	if (files[EnDeUser.XML.use] == undefined) { // file not specified in  EnDeFile.xml
		files[EnDeUser.XML.use] = ['fsetRE', 'EnDeUser.setRE(this.value)', '&#160;PHPIDS Rules:', 'PHPIDS Rules '];
	}
	var arr   = new Array();
	var labels= new Array();
	var label = '';
	var bbb   = null;
	var ccc   = null;
	for (kkk=0; kkk<bux.filters.filter.length; kkk++) {
		ccc = bux.filters.filter[kkk];
		bbb = document.createElement('OPTION');
		bbb.text  = ccc.id;
		bbb.value = ccc.rule;
		bbb.title = ccc.description;
		/* provided value for "tag" is either a string or an array of strings, stupid! */
		if (typeof(ccc.tags.tag) == 'string') {
			label = ccc.tags.tag; // a simple string
		} else {
			label = ccc.tags.tag.sort().reverse().join(', '); // most likely an array
		}
		ccc = null;
		if (label.length > 58) {
			label = label.substr(0,58) + ' ...';
		}
		bbb.text = bbb.text.substr(0,60) // use stripped comment
		// .. and push OPTION object on the stack
		if ((arr[label] == null) || (arr[label] == undefined)) {
			labels.push(label);
			arr[label] = new Array();
		}
		arr[label].push(bbb);
	}
	} // try
	catch(e){ EnDeGUI.alert('EnDeUser.XML.phpids: data failed['+kkk+']',e); }
	kkk = null;

	// now build the menu
	/* as we loop through the hash, it's automatically grouped correctly */
	try {
	bbb = document.createElement('SELECT');
	bbb.setAttribute('title', 'pattern from PHPIDS rule');
	bbb.setAttribute( click,  'document.getElementById("EnDe.RE.text").value = this.value;');
	for (kkk in labels) {
		if (kkk == 'indexOf') { continue; } // contribution to Mozilla 1.x
		label= labels[kkk];
		ccc  = document.createElement('OPTGROUP');
		ccc.setAttribute('label', label);
		while((obj = arr[label].pop())!=null) {
			ccc.appendChild(obj);
		}
		bbb.appendChild(ccc);
	}
	ccc = null; kkk = null;
	ccc = document.createElement('DIV');
	bbb.setAttribute('id',    EnDeUser.XML.use.toString());
	ccc.innerHTML = EnDeUser.XML.formatLabel( files[EnDeUser.XML.use][2] );
	ccc.appendChild(bbb);
	$('fsetRE').appendChild(ccc);
	} // try
	catch(e){ EnDeGUI.alert('EnDeUser.XML.phpids: create failed['+label+']',e); }
	finally { // reset always, also if file contained errors
		while((kkk = labels.pop())!=null) { kkk = null; };
		bux = null;
		EnDeUser.XML.use = null;
		EnDeUser.XML.cnt = 3;
		bbb = null;
		ccc = null;
		kkk = null;
	}
	if (EnDeUser.debug != 0) { _dpr('EnDeUser.XML.phpids: ============}'); }
  }; // XML.phpids

}; // XML

	// ===================================================================== //
	// checksum functions                                                    //
	// ===================================================================== //
	
this.Check  = new function() {
  this.SID  = function() { return(EnDeUser.SID() + '.Check'); };
  this.dispatch= function(tst) {
  //#? dispatcher for various checksum functions
	// var bux = $('EnDe.FT.XMLfile').value = src;
	var bux = '';
	var ccc = ' checksum: ';
/*
	var src=$('EnDe.CHK.text');
	var dst=$('EnDe.RES.text');
*/
	var src = $('EnDe.DE.text').value;
        if ($('EnDe.OPT.select').checked == true) {
		// use selection if there is one
		var kkk = EnDeGUI.selectionGet( $('EnDe.EN.text') );
		if ((kkk!=null)&&(kkk!='')) { src = kkk; }
		kkk = null;
        }
	switch (tst) {
	  case 'getISBN' : bux = EnDeCheck.ISBN.get(src);   ccc = 'get ISBN'      + ccc; break;
	  case 'getLuhn' : bux = EnDeCheck.Luhn.get(src);   ccc = 'get Luhn'      + ccc; break;
	  case 'getUPC'  : bux = EnDeCheck.UPC.get(src);    ccc = 'get UPC '      + ccc; break;
	  case 'getEAN'  : bux = EnDeCheck.EAN.get(src);    ccc = 'get EAN '      + ccc; break;
	  case 'getGLN'  : bux = EnDeCheck.GLN.get(src);    ccc = 'get GLN '      + ccc; break;
	  case 'getEFT'  : bux = EnDeCheck.EFT.get(src);    ccc = 'get EFT '      + ccc; break;
	  case 'getD5'   : bux = EnDeCheck.D5.get(src);     ccc = 'get D5  '      + ccc; break;
	  case 'isISBN'  : bux = EnDeCheck.ISBN.is(src);    ccc = 'check ISBN'    + ccc; break;
	  case 'isLuhn'  : bux = EnDeCheck.Luhn.is(src);    ccc = 'check Luhn'    + ccc; break;
	  case 'isUPC'   : bux = EnDeCheck.UPC.is(src);     ccc = 'check UPC '    + ccc; break;
	  case 'isEAN'   : bux = EnDeCheck.EAN.is(src);     ccc = 'check EAN '    + ccc; break;
	  case 'isGLN'   : bux = EnDeCheck.GLN.is(src);     ccc = 'check GLN '    + ccc; break;
	  case 'isEFT'   : bux = EnDeCheck.EFT.is(src);     ccc = 'check EFT '    + ccc; break;
	  case 'isD5'    : bux = EnDeCheck.D5.is(src);      ccc = 'check D5  '    + ccc; break;
	  case 'valISBN' : bux = EnDeCheck.ISBN.valid(src); ccc = 'validate ISBN' + ccc; break;
	  case 'valLuhn' : bux = EnDeCheck.Luhn.valid(src); ccc = 'validate Luhn' + ccc; break;
	  case 'valUPC'  : bux = EnDeCheck.UPC.valid(src);  ccc = 'validate UPC ' + ccc; break;
	  case 'valEAN'  : bux = EnDeCheck.EAN.valid(src);  ccc = 'validate EAN ' + ccc; break;
	  case 'valGLN'  : bux = EnDeCheck.GLN.valid(src);  ccc = 'validate GLN ' + ccc; break;
	  case 'valEFT'  : bux = EnDeCheck.EFT.valid(src);  ccc = 'validate EFT ' + ccc; break;
	  case 'valD5'   : bux = EnDeCheck.D5.valid(src);   ccc = 'validate D5  ' + ccc; break;
	  case 'isSSN'   : bux = EnDeCheck.SSN.is(src);     ccc = 'check SSN    ' + ccc; break;
	  case 'guess'   : EnDeGUI.cont(EnDeUser.Check.guess(src)); return false;        break;
	}
	// EnDeGUI.alert(ccc + bux);
	// $(EnDe.EN.text').value = bux;
	prompt(ccc, bux);
	return false;
  }; // dispatch
  
  this.guess     = function(src) {
  //#? guessing checksums
	var bux = '<table border="1"><tr><th colspan="4">' + src + '</th></tr>';
	bux += '<tr><td>&#160;</td><td>get()'   + '</td><td>' +'is()'                  + '</td><td>' +'valid()'                  + '</td></tr>';
	bux += '<tr><th>Luhn</th>';
	bux += '<td>' + EnDeCheck.Luhn.get(src) + '</td><td>' + EnDeCheck.Luhn.is(src) + '</td><td>' + EnDeCheck.Luhn.valid(src) + '</td></tr>';
	bux += '<tr><th>ISBN</th>';
	bux += '<td>' + EnDeCheck.ISBN.get(src) + '</td><td>' + EnDeCheck.ISBN.is(src) + '</td><td>' + EnDeCheck.ISBN.valid(src) + '</td></tr>';
	bux += '<tr><th>EFC</th>';
	bux += '<td>' + EnDeCheck.EFT.get(src)  + '</td><td>' + EnDeCheck.EFT.is(src)  + '</td><td>' + EnDeCheck.EFT.valid(src)  + '</td></tr>';
	bux += '<tr><th>UPC</th>';
	bux += '<td>' + EnDeCheck.UPC.get(src)  + '</td><td>' + EnDeCheck.UPC.is(src)  + '</td><td>' + EnDeCheck.UPC.valid(src)  + '</td></tr>';
	bux += '<tr><th>EAN</th>';
	bux += '<td>' + EnDeCheck.EAN.get(src)  + '</td><td>' + EnDeCheck.EAN.is(src)  + '</td><td>' + EnDeCheck.EAN.valid(src)  + '</td></tr>';
	bux += '<tr><th>GLN</th>';
	bux += '<td>' + EnDeCheck.GLN.get(src)  + '</td><td>' + EnDeCheck.GLN.is(src)  + '</td><td>' + EnDeCheck.GLN.valid(src)  + '</td></tr>';
	bux += '<tr><th>Verhoeff</th>';
	bux += '<td>' + EnDeCheck.D5.get(src)   + '</td><td>' + EnDeCheck.D5.is(src)   + '</td><td>' + EnDeCheck.D5.valid(src)   + '</td></tr>';
	bux += '<tr><th>GLN, ILN</th><td colspan="3">' + EnDeCheck.EAN.code(src) + '</td></tr>';
	bux += '<tr><th>EAN, IAN</th><td colspan="3">' + EnDeCheck.GLN.code(src) + '</td></tr>';
	bux += '<tr><th>IIN</th><td colspan="3">' + EnDeCheck.CC.get(src) + '</td></tr>';
	bux += '<tr><th>SSN</th><td colspan="3">' + EnDeCheck.SSN.is(src) + '</td></tr>';
	bux += '</table>';
	return(bux);
  }; // guess

}; // Check

this.IMG    = new function() {
  this.SID  = function() { return(EnDeUser.SID() + '.IMG'); };

  this.isBMP    = function(src) {
  //#? check if data is image in BMP format (lazy check only!)
	return (src.substr(0,12).match(/^BM..\x00\x00\x00\x00/)===null) ? false : true;
  }; // isBMP

  this.isTXT    = function(src) {
  //#? check if data is image in BMP format as C data structure
	return (src.substr(0,99).match(/#define\s+.*table_/)===null) ? false : true;
  }; // isTXT

  this.isPNG    = function(src) {
  //#? check if data is image in PNG format (lazy check only!)
	return (src.substr(0,12).match(/^.PNG[\r\n]/)===null) ? false : true;
	// we use a lazy check, following more strict
	//return (src.substr(0,12).match(/^\x89PNG\r\n/)===null) ? false : true;
  }; // isPNG

  this.isGIF    = function(src) {
  //#? check if data is image in GIF format (lazy check only!)
	return (src.substr(0,12).match(/^GIF89a/)===null) ? false : true;
  }; // isGIF

  this.isJPG    = function(src) {
  //#? check if data is image in JPG format (lazy check only!)
	return (src.substr(0,12).match(/^.....\x10JFIF/)===null) ? false : true;
  }; // isJPG

  this.isSVG    = function(src) {
  //#? check if data is image in SVG format (lazy check only!)
	return (src.substr(0,12).match(/DOCTYPE\s+svg/i)===null) ? false : true;
  }; // isSVG

  this.isXPM    = function(src) {
  //#? check if data is image in XPM format (lazy check only!)
// /* XPM */
// static char * ssh_xpm[] = {
	return false;
  }; // isXPM

  this.guess    = function(src) {
  //#? try to identify image format
	if (this.isGIF(src)===true) { return 'gif'; }
	if (this.isJPG(src)===true) { return 'jpg'; }
	if (this.isPNG(src)===true) { return 'png'; }
	if (this.isBMP(src)===true) { return 'bmp'; }
	if (this.isSVG(src)===true) { return 'svg'; }
	if (this.isTXT(src)===true) { return 'txt'; }
	if (this.isXPM(src)===true) { return 'xpm'; }
	return '';
  }; // guess

  this.getMIME  = function(src) {
  //#? get MIME type prefix for image data (returns data:image/x-icon for unknown format)
	var bux = 'data:image/';
	switch (src.toLowerCase()) {
	  case 'bmp': bux += 'bmp'; break;
	  case 'txt': bux += 'bmp'; break;
	  case 'gif': bux += 'gif'; break;
	  case 'jpg': bux += 'jpg'; break;
	  case 'png': bux += 'png'; break;
	  case 'svg': bux += 'svg'; break;
	  default   : bux += 'x-icon'; break;
	}
	return bux + ',';
  }; // isMIME

  this.getMIME64= function(src) {
  //#? get MIME type prefix for base64 encoded image data
	return this.getMIME(src).replace(/,$/, ';base64,');
  }; // isMIME64

}; // IMG
  
this.init      = function() {
//#? initialize user data
	// increase delay if not loaded locally; this is a guess, adapt as needed
	/* need a function to be called by global initialization function (initGUI()) 'cause
	 * conditions set there are needed here
	 */
	if (document.location.protocol.match(/file:/)==null) { EnDeUser.delay = 7000; }
	// debug, trace
	if (location.search.match(/(userdebug)/) != null) {
		EnDeGUI.trace = true;
		EnDeUser.debug = 1;
		/*
		EnDeUser.debug  = 2;
		 * use with care! 'cause it generates huge output in trace area
		 * as each line is appended to that TEXTAREA field, it's a performace problem
		 */
	}
	// boot strap
	if ((EnDeGUI.onload == true) || (EnDeGUI.fileLoad == true)) { // load files if possible or forced
		_dpr('EnDeUser.init: loading XML ..');
		EnDeUser.XML.read('EnDeOpts.xml');
		EnDeUser.XML.read('EnDeFile.xml');
		EnDeUser.XML.read('EnDeFunc.xml');
		EnDeUser.XML.read('EnDeUser.xml');
	} else {
		EnDeGUI.alert('EnDeUser','document.implementation.createDocument.load() not supported');
	}
}; // init

}; // EnDeUser
