#! /usr/bin/perl -w
#?
#? NAME
#?      $0 - XML generator
#?
#? SYNOPSIS
#?      $0 EnDeOpts.txt
#?
#? DESCRIPTION
#?      Convert tab-separated table to XML data.
#?      Also extracts comment for table format which must start with #?! .
#?
#? SEE ALSO
#?      EnDeOpts.txt
#?
#? VERSION
#?      @(#) opt2xml.pl 1.3 08/06/21 21:46:42
#?
#? AUTHOR
#?      17-jun-08 Achim Hoffmann, mailto: EnDe (at) my (dash) stp (dot) net
#?
# -----------------------------------------------------------------------------

sub entity($) {
	my $src = shift;
	$src =~ s:&:&amp;:g;
	$src =~ s:<:&lt;:g;
	$src =~ s:>:&gt;:g;
	return $src;
}

my ($size, $upper, $ishex, $prefix, $suffix, $delimiter, $key, $label, $name, $code, $desc, $data, $call, $func) = '';
$call = '';
my $cmt = 0;

open( FID, "<", $ARGV[0] ) || die "** ERROR: $!";

print <<EoXML;
<?xml version="1.0"?>
<!-- generated by opt2xml.pl 1.3 from $ARGV[0] -->
<xss>
EoXML

while (<FID>) {
	m/^#\?\!/              && do { print "\n<!--\n" if ($cmt==0); $cmt = 1; print; next; };
	m/^[ \t]*#/            && next;    # skip comments
	m/^[ \t]*$/            && do { print " -->\n\n" if ($cmt==1); $cmt = 0; next; }; # skip empty lines
	m/^[ \t]*LABEL\s+(.*)/ && do { $label = $1; next; };
	m/^[ \t]*FUNC\s+(.*)/  && do { $call  = $1; next; };
	$data = '';
	$func = '';
	($size, $upper, $ishex, $prefix, $suffix, $delimiter, $key, $code, $name, $desc) = split("\t", $_);
	chomp($desc);   # get rid of trailing newline
	$code = entity($code);
	$name = entity($name);
	$desc = entity($desc);
	$name =~ s:^(.{1,35}).*:$1:;
	$data = sprintf("<![CDATA[%s\t%s\t%s\t%s\t%s\t%s\t%s]]>", $size, $upper, $ishex, $prefix, $suffix, $delimiter, $key);
	$func = "\n\t\t<func>$call</func>\n" if ("$call" !~ /^\s*$/);
	print <<EoXML;
	<attack>
		<label>$label</label>$func
		<name>$name</name>
		<code>$code</code>
		<desc>$desc</desc>
		<opts>$data</opts>
	</attack>
EoXML
}
print "</xss>\n";
close(FID);

__END__
